%
%   htcoeff_umlrr.m   sets overall HT coeff for the UMLRR for both 
%                         forced and natural convection flow
%                   (for use in trying to estimate average Tc and Tf) 
%
%   This code is based on the same logic used in the sstemp_umlrr code. Assuming 
%   steady flow, it computes two overall HT coeffs, Ufc and Ucc. This function  
%   needs to know the mode (0 for forced or 1 for natural convection) and
%   the core inlet temp and power level. The computed HT coeffs, Ufc and Ucc, are 
%   output in W/C. The capacitance (M*cp) of the plates and the water are also output 
%   in J/C.   The input P is in W and the Tin is in oC.  Note that Pcore and
%   TinC can be either vectors or scalars...
%
%   For forced flow the Sieder-Tate correlation is used and Q = 1650 gpm (this is
%   set internally).  
%
%   Written by JRWhite, UMass-Lowell (March 2014)
%
      function [Ufc,Ucc,Cmetal,Cwater] = htcoeff_umlrr(P,TinC,mode)
%
      Pcore = P;                % core power level (W)
%
%%   define several functions for various water properties vs temperature (C)
%     k_w = thermal conductivity (W/m-K),  rho_w = density (kg/m^3)
%     mu_w = dynamic viscosity (kg/m-s),   cp_w = heat capacity (J/kg-K)
      k_w = @(T) -7.310E-06*T.^2 + 1.840E-03*T + 5.695E-01;                                   
      rho_w = @(T) 1.583E-05*T.^3 - 5.947E-03*T.^2 + 1.718E-02*T + 1000;                     
      mu_w = @(T) 3.118E-11*T.^4 - 8.702E-09*T.^3 + 9.531E-07*T.^2 - 5.427E-05*T + 1.772E-03; 
      cp_w = @(T) 3.149E-06*T.^4 - 7.683E-04*T.^3 + 7.403E-02*T.^2 - 2.851*T + 4215;  
%%   note that a lot of this code is directly from the sstemp_umlrr GUI
%
      Rtot_af = 1.0;            % resistance adjustment factor (for testing)
      g = 9.81;                 % gravitational constant (m/s^2)
%
%   set the channel geometry  --  mostly from 1991 internal memo by JRWhite
      W = 6.604/100;            % channel width (m)
      D = 0.2963/100;           % channel depth (m)
      L = 63.5/100;             % channel length (m)
      Af = W*D;                 % flow area (m^2)
      Pw = 2*(W+D);             % wetted perimeter (m)
      Dh = 4*Af/Pw;             % hydraulic diameter (m)
      Nassy = 21;               % number of fuel assemblies in core
      Ncpa = 18;                % number of coolant channels per assembly
      Nchan = Nassy*Ncpa;       % number of channels in core
%
%   set fuel plate geometry  --  mostly from 1991 internal memo by JRWhite  
      a = .0255/100;            % half thickness of fuel (m)
      b = .0380/100;            % thickness of clad (m)
      w = 6.085/100;            % active width of fuel (m)  [not used explicitly]
      H = 59.69/100;            % active height of fuel (m)
      A = H*W;                  % fuel surface area (m^2)[note use of W instead of w]
      d = 6/100;                % reflector savings (m)
      He = H+2*d;               % effective height (m)
      Neassy = 20;              % number of efective full fuel assemblies in core
      Nppa = 16;                % number of fuel plates per assembly
      Np = Neassy*Nppa;         % number of effective full fuel plates in core
      FR = 1.0;                 % radial peaking factor (1.0 for average channel)
%
%   set fuel & clad properties and compute thermal resistances
      cpc = 900;                % specific heat of clad (J/kg-K)
      volp = (2*a+2*b)*W*L;     % vol of plate (m^3)
      denc = 2700;              % density of clad (kg/m^3)
      mm = denc*volp*Np;        % mass of metal (assume all Al) (ignores side plates)
      Cmetal = cpc*mm;          % capacitance of metal in core (J/K)
      kfuel = 15;               % thermal conductivity of fuel (W/m-K)
      kclad = 180;              % thermal conductivity of aluminum (W/m-K)
      Rfuel = a/(2*kfuel);      % conductive resistance of fuel (m^2-K/W)
      Rclad = b/(kclad);        % conductive resistance of cladding (m^2-K/W)
%
      cpw = cp_w(25);           % specific heat of water at 25 C (J/kg-K)
      volchan = D*W*L;          % vol of channel (m^3)
      denw = rho_w(25);         % density of water at 25 C (kg/m^3)
      mw = denw*volchan*Nchan;  % mass of water
      Cwater = cpw*mw;          % capacitance of water in core (J/K)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Note: many of the parameters computed below can be time dependent since     %
%   Tin and P can be functions of time  --  thus, we will do a loop over time   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Note we have separate sections of code for forced and natural convection flow
    switch mode+1
%%
    case 1   % Case 1: Forced Convection Flow (pump on)
%
      Vgpm = 1650;                      % total volume flow rate through core (gpm)
      Vdott = Vgpm/(60*264.17);         % total volume flow rate through core (m^3/s)
      Ucc = zeros(size(Pcore));   Ufc = Ucc;   Nt = length(Pcore);
      for i = 1:Nt
%
%   define several key power-related parameters
        Tin = TinC(i);                  % scalar inlet temp for time point i
        Pplate = Pcore(i)/Np;           % power per plate (W) for average plate at i
        Pplate = FR*Pplate;             % heat output for plate of interest (W)
        qppave = Pplate/(2*A);          % ave heat flux (W/m^2) [see eqn. 31]
%
%   coolant flow properties (assume measured volume flow at temperature Tin)
        mdott = Vdott*rho_w(Tin);       % total mass flow rate through core (kg/s)
        Vdot = (Vdott*0.72)/Nchan;      % volumetric flow rate per channel (m^3/s)
        mdot = (mdott*0.72)/Nchan;      % mass flow rate per channel (kg/s)
        u = Vdot/Af;                    % mean fluid velocity in channel (m/s)
%
%   set fluid properties at Tbave 
%   Notes: 1. use cp at Tin for first estimate of Tbave
%          2. this scheme should converge quickly since cp is nearly constant
        Tbaveo = Tin + Pplate/(2*mdot*cp_w(Tin)); 
        err = 1;  itmax = 10;  tol = 0.0001;  it = 0;
        while err >= tol && it <= itmax
          Tbaven = Tin + Pplate/(2*mdot*cp_w(Tbaveo)); 
          err = max(abs((Tbaven-Tbaveo)/Tbaven));
          it = it+1;  Tbaveo = Tbaven;
        end
        if it > itmax, disp('WARNING:  Iteration for Tbave did not converge');  end
        Tbave = Tbaven;   
        cp = cp_w(Tbave);  k = k_w(Tbave);  mu = mu_w(Tbave);
%
%   Ucc(t) is one quantity of interest here (units of W/C)
        Ucc(i) = 2*mdot*cp*Nchan;                 
%
%   calculate the heat transfer coeff (see routine below)
        Reave = (mdot*Dh/Af)/mu;   
        h = htc(Tbave,qppave,mdot,Reave,k,cp,mu,Dh,Af);
        Rconv = 1/h;  
%
%   compute total thermal resistance (Rtot_af - resistance adjustment factor)
        Rtot = (Rfuel+Rclad+Rconv)*Rtot_af;   % units are m^2-C/W
        Rtot = Rtot/(2*A*Np);                 % units are C/W           
        Ufc(i) = 1/Rtot;                      % units of W/C
      end
%%   
    case 2   %  Case 2: Natural Convection Flow (pump off)
%
%   estimate of channel mass flow rate in natural convection mode
      Vdoto = (1650/200)/(60*264.17*Nchan); % initial guess at vol flow rate in channel
      Ucc = zeros(size(Pcore));   Ufc = Ucc;   Nt = length(Pcore);
      for i = 1:Nt
%
%   define several key power-related parameters
        Tin = TinC(i);                  % scalar inlet temp for time point i
        Pplate = Pcore(i)/Np;           % power per plate (W) for average plate at i
        Pplate = FR*Pplate;             % heat output for plate of interest (W)
        qppave = Pplate/(2*A);          % ave heat flux (W/m^2) [see eqn. 31]
        if i == 1
          mdoto = Vdoto*rho_w(Tin);     % initial guess at channel mdot (kg/s)
        else
          mdoto = mdot;                 % use mdot from previous time step as guess
        end
        err1 = 1;  itmax1 = 20;  tol1 = 0.0001;  it1 = 0; 
        while err1 >= tol1 && it1 <= itmax1
%   first compute Tbave for the given mdot
          Tbaveo = Tin + Pplate/(2*mdoto*cp_w(Tin)); 
          err2 = 1;  itmax2 = 10;  tol2 = 0.0001;  it2 = 0;
          while err2 >= tol2 && it2 <= itmax2
            Tbaven = Tin + Pplate/(2*mdoto*cp_w(Tbaveo)); 
            err2 = abs((Tbaven-Tbaveo)/Tbaven);
            it2 = it2+1;  Tbaveo = Tbaven;
          end
          if it2 > itmax2 && it1 > 1, disp('WARNING:  Iteration for Tbave in mdot loop did not converge');  end
          Tbave = Tbaven;  
          cp = cp_w(Tbave);  k = k_w(Tbave);  rho = rho_w(Tbave);  mu = mu_w(Tbave);
%   now calc new mdot by equating buoyant and friction forces in channel
          drho = rho_w(Tin)-rho;                % average change in density between pool & channel (kg/m^3)
          dP = drho*g*L;                        % change in pressure due to buoyancy (N/m^2)
          hL = dP/(rho*g);                      % head loss required for steady state flow (m)
          Re = (mdoto*Dh/Af)/mu;                % ave Reynolds number 
          if Re <= 2300
            f = 96/Re;   K = 0.5+2.0;           % friction factor for laminar flow
          else
            f = 0.3164/Re^0.25;  K = 0.5+1.05;  % friction factor for turbulent flow
          end
          u =  sqrt((2*g*hL)/(((f*L)/Dh)+K));   % computed velocity
          mdotn = u*rho*Af;                     % computed channel mass flow rate 
          err1 = max(abs((mdotn-mdoto)/mdotn));
          it1 = it1+1;  mdoto = mdotn;
        end
        if it1 > itmax1, disp('WARNING:  Iteration for mdot did not converge');  end
        mdot = mdotn;
%
%   Ucc(t) is one quantity of interest here (desire units of W/C)
        Ucc(i) = 2*mdot*cp*Nchan;                 
%
%   calculate the heat transfer coeff (see routine below)
        Reave = (mdot*Dh/Af)/mu;   
        h = htc(Tbave,qppave,mdot,Reave,k,cp,mu,Dh,Af);
        Rconv = 1/h;  
%
%   compute total thermal resistance (Rtot_af - resistance adjustment factor)
        Rtot = (Rfuel+Rclad+Rconv)*Rtot_af;   % units are m^2-C/W
        Rtot = Rtot/(2*A*Np);                 % units are C/W           
        Ufc(i) = 1/Rtot;                      % units of W/C
      end
    end   %  end of function

%%
      function h = htc(Tb,qpp,mdot,Reave,k,cp,mu,Dh,Af)
%
%   this routine is called for each time point (i.e. scalar quantities)
      if Reave <= 2300   % laminar flow
        Nu = (8.23+7.54)/2;             % Nusselt number from Incropera Table 8.1
        h = (Nu*k)/Dh;                  % convection HT coefficient (W/m^2-K)
      else               % turbulent flow
        Re = (mdot*Dh/Af)/mu;           % Reynolds number 
        Pr = mu*cp/k;                   % Prandtl number           
        Nu = 0.023*Re^(0.8)*Pr^(0.4);   % Nusselt number (Dittus-Boelter)
        h = (Nu*k)/Dh;                  % heat transfer coefficient (W/m^2-K)
        Rconv = 1/h;                    % convection resistance (m^2-K/W)
        Tc = Tb + qpp*Rconv;            % clad surface temperatures (C)
%   for the Seider Tate correlation, we use DB as guess and iterate on Tc 
        err = 1;  itmax = 10;  tol = 0.0001;  it = 0;  Tco = Tc;    
        mu_w = @(T) 3.118E-11*T^4 - 8.702E-09*T^3 + 9.531E-07*T^2 - 5.427E-05*T + 1.772E-03; 
        while err >= tol && it <= itmax
          mus = mu_w(Tco);                   
          Nu = 0.027*Re^(.8)*Pr^(1/3)*(mu/mus)^(0.14); % Nusselt number (Sieder-Tate)
          h = (Nu*k)/Dh;                     
          Rconv = 1./h;                      
          Tcn = Tb + qpp*Rconv;             
          err = abs((Tcn-Tco)/Tcn);
          it = it+1;  Tco = Tcn;
        end
        if it > itmax, disp('WARNING:  Iteration for Tc did not converge');  end
      end
%  
%   end of function
