%
%   MODEL_TEST_FDBK.M   Test simulations for a UMLRR dynamics model 
%                      that includes temperature and xenon feedbacks
%
%   A series of forced and natural convection flow experiments was performed in the 
%   UMLRR over the the last several years to show that the inherent feedback 
%   reactivity within the system could compensate for a positive reactivity addition 
%   and, with xenon buildup, even shut down the reactor without operator control.
% 
%   The experimental procedure was pretty simple in all cases: that is, the reactor 
%   was initially at low power (about 5 kW) and then a positive reactivty was 
%   inserted by moving the RegBlade out (usually between 1 - 3 inches), and then 
%   the system was left to stabilize itself without additional reactivity control
%   by the operators.  
%
%   This file uses those expts to test a simulation model that includes a temperature
%   and xenon reactivtiy feedback model for the UMLRR.  Both forced and natural
%   convection cases can be treated...
%
%   File prepared by J. R. White, UMass-Lowell (March 2014)
%   Reviewed with no substantial changes   (JRWhite, Nov. 2015, March 2018)

      clear all, close all, nfig = 0;
%%
%   build in the option to use calculated or optimum feedback coeff for fuel temp
      opt_alfTf = 'optimum';             % this is the default
      iskip = 1;                         % if iskip > 0 then this section is skipped
      if iskip == 0
        iopt = menu('Select feedback option for Fuel Temp coeff.', ...
                    'Use optimum value from 2007 study: alfTf = -7.7e-5 Dk/k',...
                    'Use the calculated value from M. Pike''s MS Thesis: -1.7e-5 Dk/k');
        if iopt == 2, opt_alfTf = 'calculated';  end
      end
%%
%   select the case of interest
      isw = menu('Select the test case of interest', ...
                 '  Forced flow test from August 20, 2012  ', ...
                 '  Natural Convection test from August 20, 2007 ');
      switch isw
% forced flow cases  (mode = 0)
        case 1    % August 20, 2012
          fname = 'model_test_forced_flow_082012.dat';
          zlocs = [7.88 9.45];   tstrt = 61;  tdur = 150*60-tstrt;  yr = 2013;  mode = 0;
% natural convection flow cases (mode = 1)
        case 2   % August 20, 2007
          fname = 'model_test_natconv_082007.dat';
          zlocs = [9.62 11.74];  tstrt = 24;   tdur = 80*60-tstrt;  yr = 2007;  mode = 1;
      end
      boi = 5;    % RegBlade is the blade of interest (BOI)
%
%   read experimental data
      [data,tags,exptdate,starttime] = read_datfile(fname); 
%
%   plot some data to view reactor operation during the run
      tem = data(:,1)/60;     % time vector for expt (minutes)
      tunits = 'minutes';     % time units for plots
      titl = 'Model\_Test';   % for use in title of plots
      nfig = plot_PTB(tem,data,tags,exptdate,nfig,titl,tunits);  
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  now with P(t), lets generate measured rho(t) with inverse kinetics  %
%     (note that time and all the data are offset by tstrt seconds)    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      Nt = length(data(:,1));  trange = tstrt+1:Nt;     % time range of interest
      tes = data(trange,1)-tstrt;                       % time starting at t=0 sec 
      iLPwrComb = gettagloc('LPwrComb',tags);           % loc of LPwrComb signal
      LPwrComb = data(trange,iLPwrComb);                % measured frac of full power
      iLogPower = gettagloc('LogPower',tags);           % loc of LPwrComb signal
      LogPower = data(trange,iLogPower);                % measured frac of full power
      [Be,B,Gt,lam] = kinetics_data();                  % kinetics data 
      Co = B*LPwrComb(1)./(lam*Gt);  dt = 1;            % initial precursor conc
      rhotot_meas = invkin_sr(LPwrComb,tes,lam,Co,dt,Be,Gt,B); % total rho from invkin
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Now do a Simulation of this Reactor Secquence  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   set specifics for this case
      F2C = @(T) (T-32)/1.8;           % convert oF to oC
      C2F = @(T) 1.8*T+32;             % convert oC to oF
      PokW = LPwrComb(1)*10;           % initial power level [1MW/100)*1000 -> kW]
      tT = tes;                        % time vector for experimental Tin
      iTin = gettagloc('CoreTin ',tags); % loc of measured core inlet temp (downward)
      if mode == 1,  iTin = gettagloc('CoreTout',tags);  end  % flow is upward
      Tint = F2C(data(trange,iTin));   % Tin(t) in oC  
      Tino = Tint(1);                  % initial inlet temperature (oC)
%%
%   set initial conditions and other parameters
      [Be,B,Gt,lam,td,S,kappa,nu] = kinetics_data(); % kinetics data for simulations
      rho0 = 0;  Q = 0;       % assume critical and no source
      Po = PokW*1000;         % initial power in W
      [Ufc,Ucc] = htcoeff_umlrr(Po,Tino,mode);  % calc HT coeffs at initial state    
      Tco = Tino + Po/Ucc;    % initial ave coolant temp (oC)
      Tfo = Tco + Po/Ufc;     % initial ave fuel temp (oC)
      Tcr = Tco;  Tfr = Tfo;  % set reference temps for calc of temp feedbacks
      Co = B*Po./(lam*Gt);    % initial precursor conc
      Io = 0; Xo = 0;         % initial I-135 & Xe-135 concentrations 
      xo = [Po Co Tfo Tco Io Xo]';  % initial condition for state vector (11 states)
      rate = 40/60;           % rate of RegBlade change (inches/sec)
      cfac = 0.88;            % flux correction factor
%% 
%   simulate a single reactivity transient
      [t,x,z,rhoex] = simstep_fdbk(xo,rho0,Q,zlocs,0,tdur,rate, ...
                              tT,Tint,Tcr,Tfr,cfac,mode,boi,yr,opt_alfTf);
      Pn = x(:,1);  Tf = x(:,8);  Tc = x(:,9);  Xe = x(:,11);
%
%   now plot the results (1 sec delay is added to acct for the measurement delay)
      tp = (t+1)/60;  tem = tes/60;
      nfig = nfig+1;  figure(nfig)
      subplot(2,1,1),plot(tp,rhoex*100,'r-','LineWidth',2),grid on
      title(['Model\_Test: Simulated \rho(t) & Blade Position for Expt on ',exptdate])
      ylabel('Reactivtiy (%\Deltak/k)')
      subplot(2,1,2),plot(tp,z,'b-','LineWidth',2),grid on
      raxis = axis;  raxis(1) = -1;  axis(raxis);
      xlabel('Time (min)'), ylabel('Blade Position (inches out)')
%
      nfig = nfig+1;  figure(nfig)
      subplot(2,1,1),plot(tp,Pn/1000,'r-','LineWidth',2),grid on
      title(['Model\_Test: Simulated P(t) & Blade Position for Expt on ',exptdate])
      ylabel('Power Level (kW)')
      subplot(2,1,2),plot(tp,z,'b-','LineWidth',2),grid on
      raxis = axis;  raxis(1) = -1;  axis(raxis);
      xlabel('Time (min)'), ylabel('Blade Position (inches out)')
%
      nfig = nfig+1;  figure(nfig)
      plot(tp,Tf,'r-',tp,Tc,'b--','LineWidth',2),grid on
      title(['Model\_Test: Simulated T_f(t) and T_c(t) for Expt on ',exptdate])
      xlabel('Time (min)'),ylabel('Temperature (^oC))')
      legend('Fuel Temp','Coolant Temp')
%
      nfig = nfig+1;  figure(nfig)
      plot(tp,Pn/1000,'r-',tem,LogPower*10,'g-.',tem,LPwrComb*10,'b--','LineWidth',2),grid on
      title(['Model\_Test: Simulated vs Measured P(t) for Expt on ',exptdate])
      xlabel('Time (min)'),ylabel('Power Level (kW)')
      legend('Simulated P(t)','Measured LogPower','Measured LPwrComb')
%
%%
%   now let's reconstruct the reactivity feedback components
      Tunits = 'oC';   % options for reactivity coeffs (opt_alfTf set at top of file)
      [alfTf,alfTc] = temp_coeff(Tc,Tunits,opt_alfTf);% reactivity coeffs (Dk/k/oC)
      rhoTf = alfTf*(Tf - Tfr);                       % fuel temp feedback (Dk/k)
      rhoTc = alfTc.*(Tc - Tcr);                      % coolant temp feedback (Dk/k)
% 
      [dci,yi,dcx,yx,sigax,denom_rhox,sigfeff,sigf2,flxfull] = xenon_data();
      rhoX = Xe*sigax/sigf2/denom_rhox;  % Xe rho         
%
      rhof = rhoTc+rhoTf+rhoX;           % feedback rho 
      rhotot = rhoex-rhof;               % total rho
%
%   plot reactivity components
      nfig = nfig+1;  figure(nfig);  
      plot(tem,rhotot_meas*100,'g-',tp,rhoex*100,'m-',tp,-rhoX*100,'r--',tp,-rhoTf*100,'c-.', ...
           tp,-rhoTc*100,'b-.',tp,rhotot*100,'k-','LineWidth',2),grid on
      raxis = axis;  raxis(1) = -1;  axis(raxis);
      title(['Model\_Test: Simulated \rho(t) Components for Expt on ',exptdate])
      xlabel('Time (minutes)'),ylabel('Reactivity Worth (% \Deltak/k)')
      legend('\rho_{tot} measured','\rho_{ext} worth curve','\rho_{Xe} model', ...
             '\rho_{Tf} model','\rho_{Tc} model','\rho_{tot} model','Location','NorthEast')
%
%   end of program