%
%   RegBlade_Sim_FDBK.M   Test simulation for a UMLRR dynamics model 
%                         including temperature and xenon feedbacks
%
%   This routine does a test simulation by moving the RegBlade and computing the 
%   P(t) profile with feedbacks.  It also allows an input Tin(t).  This case uses
%   only two arbitrary steps, but any number can be added for a specific case of 
%   interest.  Several parameters (mode, Po, Tint, rate, and, of course, zlocs for 
%   each step) should be set by the user for the case of interest.
%
%   File prepared by J. R. White, UMass-Lowell (last update: April 2017)
%

      clear all, close all, nfig = 0;
%%
%   set specifics for this case
      mode = menu('Select Flow Scenario', ...
                   '     Forced Flow      ','  Natural Convection Flow ');
      mode = mode-1;  % 0 - forced flow   and   1 - natural convection flow
      opt_alfTf = 'optimum';    % use fuel temp coeff from 2007 optimization study
      boi = 5;  yr = 'Jan2016'; % use the RegBlade curves from 2016
      Po = 5000;                % initial power level (W)
      tT = [0 300*60]; Tint = [25 25];   % arbitrary Tin(t) in oC  (for testing)
      Tino = Tint(1);           % initial inlet temperature (oC)
      tstrt = 0;                % start time for simulation 

%%
%   set initial conditions and other parameters
      [Be,B,Gt,lam,td,S,kappa,nu] = kinetics_data(); % kinetics data for simulations
      rho0 = 0;   Q = 0;      % assume critical and no source
      [Ufc,Ucc] = htcoeff_umlrr(Po,Tino,mode);  % calc initial HT coeffs (W/C)     
      Tco = Tino + Po/Ucc;    % initial ave coolant temp (oC)
      Tfo = Tco + Po/Ufc;     % initial ave fuel temp (oC)
      Tcr = Tco;  Tfr = Tfo;  % set reference temps for calc of temp feedbacks
      Co = B*Po./(lam*Gt);    % initial precursor conc
      Io = 0; Xo = 0;         % initial I-135 & Xe-135 concentrations 
      xo = [Po Co Tfo Tco Io Xo]';  % initial condition for state vector (11 states)
      rate = 40/60;           % rate of RegBlade change (inches/sec)
      cfac = 0.88;            % flux correction factor
%%
%   do a series of blade movements (nothing fancy, just repeat same section of code)
%
      rr = [];  tt = [];  zz = [];  PP = [];  TTf = [];  TTc = [];  XXe = [];   
%   Step #1
      zlocs = [12.0 14.0];  tdur = 40*60;
      [t,x,z,rhoex] = simstep_fdbk(xo,rho0,Q,zlocs,tstrt,tdur,rate, ...
                              tT,Tint,Tcr,Tfr,cfac,mode,boi,yr,opt_alfTf);
      tstrt = t(end);  rho0 = rhoex(end);  xo = x(end,:)';
      Pn = x(:,1);  Tf = x(:,8);  Tc = x(:,9);  Xe = x(:,11);
      rr = [rr; rhoex];  tt = [tt; t];  zz = [zz; z];  
      PP = [PP; Pn];  TTf = [TTf; Tf];  TTc = [TTc; Tc];  XXe = [XXe; Xe];
%
%   Step 2
      zlocs = [14.0 11.0];   tdur = 20*60;     
      [t,x,z,rhoex] = simstep_fdbk(xo,rho0,Q,zlocs,tstrt,tdur,rate, ...
                              tT,Tint,Tcr,Tfr,cfac,mode,boi,yr,opt_alfTf);
      tstrt = t(end);  rho0 = rhoex(end);  xo = x(end,:)';
      Pn = x(:,1);  Tf = x(:,8);  Tc = x(:,9);  Xe = x(:,11);
      rr = [rr; rhoex(2:end)];  tt = [tt; t(2:end)];  zz = [zz; z(2:end)];  
      PP = [PP; Pn(2:end)];    TTf = [TTf; Tf(2:end)];  
      TTc = [TTc; Tc(2:end)];  XXe = [XXe; Xe(2:end)];
%
%   now plot the full sequence
      mtxt = '(Forced Flow Mode)';
      if mode == 1,  mtxt = '(Natural Convection Mode)';  end
      nfig = nfig+1;  figure(nfig)
      subplot(2,1,1),plot(tt/60,rr*100,'r-','LineWidth',2),grid on
      raxis = axis;  raxis(1) = -1;  axis(raxis);
      title(['Simulated \rho(t) and Blade Position vs Time ',mtxt])
      ylabel('Reactivtiy (%\Deltak/k)')
      subplot(2,1,2),plot(tt/60,zz,'b-','LineWidth',2),grid on
      raxis = axis;  raxis(1) = -1;  axis(raxis);
      xlabel('Time (min)'), ylabel('Blade Position (inches out)')
%
      nfig = nfig+1;  figure(nfig)
      subplot(2,1,1),plot(tt/60,PP/1000,'r-','LineWidth',2),grid on
      raxis = axis;  raxis(1) = -1;  axis(raxis);
      title(['Simulated P(t) and Blade Position vs Time ',mtxt])
      ylabel('Power Level (kW)')
      subplot(2,1,2),plot(tt/60,zz,'b-','LineWidth',2),grid on
      raxis = axis;  raxis(1) = -1;  axis(raxis);
      xlabel('Time (min)'), ylabel('Blade Position (inches out)')
%
      nfig = nfig+1;  figure(nfig)
      plot(tt/60,TTf,'r-',tt/60,TTc,'b--','LineWidth',2),grid on
      title(['Simulated T_f(t) and T_c(t) vs Time ',mtxt])
      xlabel('Time (min)'),ylabel('Temperature (^oC))')
      legend('Fuel Temp','Coolant Temp')
%
      nfig = nfig+1;  figure(nfig)
      plot(tt/60,PP/1000,'r-','LineWidth',2),grid on
      title(['Simulated P(t) vs Time ',mtxt])
      xlabel('Time (min)'),ylabel('Power Level (kW)')
%
%%  now let's reconstruct the reactivtiy feedback components
      Tunits = 'oC';   % options for reactivity coeffs (opt_alfTf set at top of file)
      [alfTf,alfTc] = temp_coeff(TTc,Tunits,opt_alfTf);% reactivity coeffs (Dk/k/oC)
      rhoTf = alfTf*(TTf - Tfr);         % fuel temp feedback
      rhoTc = alfTc.*(TTc - Tcr);        % coolant temp feedback
% 
      [dci,yi,dcx,yx,sigax,denom_rhox,sigfeff,sigf2,flxfull] = xenon_data();
      rhoX = XXe*sigax/sigf2/denom_rhox; % Xe rho         
%
      rhof = rhoTc+rhoTf+rhoX;           % feedback rho 
      rhotot = rr-rhof;                  % total rho
%
%   plot reactivtiy components
      nfig = nfig+1;  figure(nfig);
      plot(tt/60,rr*100,'m-',tt/60,-rhoX*100,'r--',tt/60,-rhoTf*100,'c-.', ...
           tt/60,-rhoTc*100,'b-.',tt/60,rhotot*100,'k-','LineWidth',2),grid on
      title(['Reactivity Components from Simulation ',mtxt])
      xlabel('Time (minutes)'),ylabel('Reactivity Worth (% \Deltak/k)')
      legend('\rho_{ext} worth curve','\rho_{Xe} model', ...
             '\rho_{Tf} model','\rho_{Tc} model','\rho_{tot} model','Location','NorthEast')


%
%   end of program