%
%   temp_coeff.m  Routine to set the temperature reactivtiy coeffs for use 
%                   in quantifying the feedback models within the UMLRR
%
%   Note: A fuel reactivtiy coeff of -1.7e-5 Dk/k-C from Michael Pike's thesis gives
%   very poor results in all the simulations.  This is actually consistent with 
%   observations made back in summer 2007 when we were doing some similar feedback 
%   tests.  However, I believe the problem is more with the calculation of the fuel 
%   temperatures (i.e. the thermal resistance model) more than it is with the fuel 
%   reactivity coeff.  In any case, one of them is clearly wrong, so I will modify 
%   the reactivity coeff here and leave the mathematical model alone (since I really
%   don't know how to adjust the resistances).  In 2007, we did an (undocumented)
%   optimization study and found that a value of alfTf = -7.7e-5 Dk/k-C gave pretty
%   good results for a couple of runs during that summer.  Thus let's start with
%   this as our reference value.  However, with this adjustment, we now clearly have 
%   an empirical model (i.e. coeffs based on experimental observation)... 
%
%   Summary reference values:
%     alfTtot = 5.9e-5   --  ave total temp coeff (Dk/k-C) (expt 012213)
%     alfTfMP = 1.7e-5   --  fuel temp coeff (Dk/k-C)  (Michael Pike's MS Thesis)
%     alfTcref = 4.2e-5  --  ref coolant temperature coeff (Dk/k-C) 
%     alfTf = 7.7e-5     --  optimum fuel temp coeff (Dk/k-C) (summer 2007)
%
%                                                              (JRWhite March 2013)
%
%   Note: For natural convection the coolant temperature change can be significant, so
%   we definitely need to take into account the temperature dependence of the coolant 
%   temp coeff.
%   To do this, we will use the temperature dependence determined from the 01/22/13
%   pool cooldown experiment, but adjust the value with a bias so that it gives the
%   nominal value close to alfTcref = 4.2e-5 Dk/k-C at T = 25 C = 77 F which was in  
%   the middle of the range tested.  We can do this same thing for any case (forced
%   or natural convection), although it probably won't make much difference in the
%   forced flow cases.
%
%   Note: I decided to build in some options for the temperature units and for the
%   the selection of which alfTf to use.  Thus,
%      if Tunits = 'oF', then alf is in Dk/k per oF
%      if Tunits = 'oC', then alf is in Dk/k per oC
%      if opt_alfTf = 'optimum', use alfTf from optimization study (summer 2007)
%      if opt_alfTf = 'calculated', use alfTf from Michael Pike's MS Thesis
%
%   Original written by JRWhite, UMass-Lowell (March 2013)
%   Mod#1:  Added options noted above  (JRWhite, March 2014)
%
%
       function [alfTf,alfTc] = temp_coeff(Tc,Tunits,opt_alfTf)
%
%   remove the fuel temp coeff from the total to get eqn fot alfTc
      bias = 1.7e-5;             % remove alfTfMP to get alfTc = alfTcref at T=25C  
      fT = @(T) -2.88e-6*T + 1.23e-5 + bias;  % equation from expt 012213 (T in oC)
%
%   select the fuel temp coeff to use
      if strcmp(opt_alfTf,'optimum')
        alfTf = 7.7e-5;          % fuel temp coeff (Dk/k-C) (summer 2007)
      end
      if strcmp(opt_alfTf,'calculated')
        alfTf = 1.7e-5;          % fuel temp coeff (Dk/k-C) (Michael Pike's thesis)
      end
%
%   adjust for proper units
      if strcmp(Tunits,'oC')
        alfTc = abs(fT(Tc));     % temp dependent coolant temp coeff (Dk/k-C) 
      end
      if strcmp(Tunits,'oF')
        alfTf = alfTf/1.8;       % fuel temp coeff (Dk/k-F) (summer 2007)
        TcC = (Tc-32)/1.8;       % core coolant temp in oC
        alfTc = abs(fT(TcC))/1.8;% temp dependent temp coeff (Dk/k-F) 
      end
%
%   end of function
